#!/usr/bin/perl


=begin

	(filename)

	reown.plx

	(copyright)

	Copyright (c) 2013 Alexej Magura

	This file is part of Reform.

	Reform is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Reform is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Reform.  If not, see <http://www.gnu.org/licenses/>.

=cut

use strict;
use warnings;

use File::Basename qw(basename);
use File::Temp;
use File::Compare;
use Getopt::Long;
use Text::Diff qw(diff);
use Config::Simple;


Getopt::Long::Configure("bundling");


# Getopt Switch-Flags

my $help=0;
my $version=0;
my $verbose=0;
my $printOnly=0;


# Getopt Value Vars

my $version_string = '0.2.2';
my $name = "";
my $email = "";
my $config = "";

GetOptions ("help|h" => \$help,
	    "version" => \$version,
	    "dry-run|d" => \$printOnly,
	    "name|n=s" => \$name,
	    "email|e=s" => \$email,
	    "verbose|v" => \$verbose,
	    "config=s" => \$config)
    or exit 2;

if ( $help) {

    printf ( "usage: %s [option]... <file>... \n", basename($0));

    my @help_list = ("\n",
		     "  -h, --help\t\tprint this message and exit\n",
		     "  -v, --verbose\t\tprint (unified) diff of the before and after file(s)\n",
		     "  -d, --dry-run\t\tprint, but do not commit, the after file(s)\n",
		     "  -e, --email=<email>\tspecify new Maintainer's email\n",

		     "  -n, --name=<name>\tspecify new Maintainer's name\n",
		     "      --version\t\tprint version and exit\n\n");

		 map { print $_ } @help_list;

}
elsif ( $version) {

    printf ( "%s\n", $version_string);

    exit 0;
}



# Config parsing

my $cfg;

if ( $config eq "") {

    $config = '/etc/reform/reown.conf';

}

if ( -e $config) {
    
   $cfg = new Config::Simple($config);

   $name  = $cfg->param('name');
   $email = $cfg->param('email');

}

sub main {

    my $MAINTAINER=0;
    my $fh = File::Temp->new( SUFFIX => '.tmp', TEMPLATE => 'reown-XXXXXXX', DIR => '/tmp', UNLINK => 0);
    my $fname = $fh->filename;

    if ( not ( @_)) {

	exit 3;

    }

    foreach my $file ( @_) {

	
	if ( not (-e $file)) {

	    exit 1;

	}
	elsif ( not (-R $file)) {

	    exit 5;
	}

	open ( FILE_a, "<$file");
	open ( FILE_b, ">$fname");

	while (<FILE_a>) {

	    if ( $_ =~ /^(\s*)#(\s*)Maintainer(\s*):(\s*)(.*)$/) {

		if (( "$name" eq "") and ( "$email" eq "") and ( "$5" eq "")) {

		    $MAINTAINER=1;
		}
		elsif ( "$5" eq "$name $email") {

		    $MAINTAINER=1;
		    # print $MAINTAINER."\n";

		}

	    }
	}

	seek(FILE_a, 0, 0);
	# print $MAINTAINER."\n";
	
	if ( $MAINTAINER == 0) {
	    
	    print FILE_b "# Maintainer: $name $email\n";

	}

	while (<FILE_a>) {

	    if ( $MAINTAINER == 0) {
		
		$_ =~ s/^(\s*)#(\s*)Maintainer(\s*):/$1#$2Contributor$3:/g;

	    }

	    print FILE_b "$_";

	}

	close(FILE_a); 
	close(FILE_b);

	if ( compare($file, $fname) == 0) {

	    unlink $fname;

	    exit 4;

	}

	open(FILE_b, "<$fname");

	if ( $verbose) {

	    print diff $file => $fname;

	}

	if ( $verbose and $printOnly) {

	    print "\e[0;34m~\e[0;0m\n" x 10;
	}
	
	if ( $printOnly) {

	    while (<FILE_b>) {

		print "$_";

	    }

	    unlink $fname;

	    exit 0;

	}

	open(FILE_a, ">$file");

	while (<FILE_b>) {

	    print FILE_a "$_";

	}

	close(FILE_a);
	close(FILE_b);

	unlink $fname;

    }
}


main(@ARGV);


=begin TODO 

    [1] - Add support for a configuration file, like the kind used by burp, so that users won't have to be constantly passing their name and email as arguments to reown.plx;

=cut
